/******************************************************************************

  This source file is part of the tomviz project.

  Copyright Kitware, Inc.

  This source code is released under the New BSD License, (the "License").

  Unless required by applicable law or agreed to in writing, software
  distributed under the License is distributed on an "AS IS" BASIS,
  WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
  See the License for the specific language governing permissions and
  limitations under the License.

******************************************************************************/

#include "ModuleScaleCubeWidget.h"
#include "ui_ModuleScaleCubeWidget.h"

#include <QDoubleValidator>
#include <QTextStream>

namespace tomviz {

ModuleScaleCubeWidget::ModuleScaleCubeWidget(QWidget* parent_)
  : QWidget(parent_), m_ui(new Ui::ModuleScaleCubeWidget)
{
  m_ui->setupUi(this);
  m_ui->leSideLength->setValidator(new QDoubleValidator(this));

  connect(m_ui->chbAdaptiveScaling, SIGNAL(toggled(bool)), this,
          SIGNAL(adaptiveScalingToggled(const bool)));
  connect(m_ui->leSideLength, &QLineEdit::editingFinished, this,
          [&] { sideLengthChanged(m_ui->leSideLength->text().toDouble()); });
  connect(m_ui->chbAnnotation, SIGNAL(toggled(bool)), this,
          SIGNAL(annotationToggled(const bool)));
}

ModuleScaleCubeWidget::~ModuleScaleCubeWidget() = default;

void ModuleScaleCubeWidget::setAdaptiveScaling(const bool choice)
{
  m_ui->chbAdaptiveScaling->setChecked(choice);
}

void ModuleScaleCubeWidget::setSideLength(const double length)
{
  m_ui->leSideLength->setText(QString::number(length));
}

void ModuleScaleCubeWidget::setAnnotation(const bool choice)
{
  m_ui->chbAnnotation->setChecked(choice);
}

void ModuleScaleCubeWidget::setLengthUnit(const QString unit)
{
  m_ui->tlLengthUnit->setText(unit);
}

void ModuleScaleCubeWidget::setPosition(const double x, const double y,
                                        const double z)
{
  QString s;
  QTextStream(&s) << "(" << QString::number(x, 'f', 4) << ", "
                  << QString::number(y, 'f', 4) << ", "
                  << QString::number(z, 'f', 4) << ")";
  m_ui->tlPosition->setText(s);
}

void ModuleScaleCubeWidget::setPositionUnit(const QString unit)
{
  m_ui->tlPositionUnit->setText(unit);
}

void ModuleScaleCubeWidget::onAdaptiveScalingChanged(const bool state)
{
  emit adaptiveScalingToggled(state);
}

void ModuleScaleCubeWidget::onSideLengthChanged(const double length)
{
  emit sideLengthChanged(length);
}

void ModuleScaleCubeWidget::onAnnotationChanged(const bool state)
{
  emit annotationToggled(state);
}
}
